<?php
namespace App\Form\Activity;

use App\Entity\ActivityEntity;
use App\Entity\ActorEntity;
use App\Enum\StatusEnum;
use App\Form\AbstractFormManager;
use App\Repository\ActivityRepository;
use Doctrine\Common\Collections\Criteria;
use Doctrine\ORM\Query;
use Doctrine\ORM\Query\Expr\Join;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\DateType;
use Symfony\Component\Form\Extension\Core\Type\ResetType;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;

class ActivityFilterForm extends AbstractFormManager
{

    /** @var array List of actors */
    protected $actors = [];

    /**
     * {@inheritdoc}
     */
    public function __construct(FormBuilderInterface $formBuilder, array $data = [], bool $usesToken = true)
    {
        $this->actors = $data['actors'];
        $formBuilder->setData([
            'actor' => null,
            'unassigned' => false,
            'followed' => false,

            'status' => null,
            'active' => false,
            'notClosed' => true,

            'endAfter' => null,
            'startBefore' => null,
        ]);
        parent::__construct($formBuilder, $data, $usesToken);
    }

    /**
     * Get activities
     *
     * @param ActivityRepository $activityRepository
     * @return array
     */
    public function getActivities(ActivityRepository $activityRepository): array
    {
        $data = $this->getData();

        // Create query
        $query = $activityRepository->createQueryBuilder('activity');

        // Create filter
        $criteria = new Criteria();

        // Filter actors
        if ($data['followed']) {
            $query->join(ActorEntity::class, 'actor', Join::WITH, 'activity.actor = actor');
            $criteria->andWhere($criteria->expr()
                ->eq('actor.followed', true));
        }
        if (!is_null($data['actor'])) {
            $criteria->andWhere($criteria->expr()
                ->eq('activity.actor', $data['actor']));
        }
        if ($data['unassigned']) {
            $criteria->andWhere($criteria->expr()
                ->isNull('activity.actor'));
        }

        // Filter status
        if (!is_null($data['status'])) {
            $criteria->andWhere($criteria->expr()
                ->eq('activity.status', $data['status']));
        }
        if ($data['active']) {
            $criteria->andWhere($criteria->expr()
                ->in('activity.status', StatusEnum::ACTIVE_STATUS));
        }
        if ($data['notClosed']) {
            $criteria->andWhere($criteria->expr()
                ->notIn('activity.status', StatusEnum::CLOSED_STATUS));
        }

        // Filter dates
        if (!is_null($data['endAfter'])) {
            $criteria->andWhere($criteria->expr()
                ->orX($criteria->expr()
                ->isNull('activity.realEndDate'), $criteria->expr()
                ->gt('activity.realEndDate', $data['endAfter'])));
        }
        if (!is_null($data['startBefore'])) {
            $criteria->andWhere($criteria->expr()
                ->orX($criteria->expr()
                ->isNull('activity.realStartDate'), $criteria->expr()
                ->lt('activity.realStartDate', $data['startBefore'])));
        }

        $query->addCriteria($criteria);

        return $query->getQuery()->execute([], Query::HYDRATE_OBJECT);
    }

    /**
     * {@inheritdoc}
     */
    public function getTemplate(): string
    {
        return '_includes/html/form/activity/filter.html.twig';
    }

    /**
     * {@inheritdoc}
     */
    protected function addFields($formBuilder, $options): void
    {
        // Actor filters
        $formBuilder->add('actor', EntityType::class, [
            'class' => ActorEntity::class,
            'choices' => $this->actors,
            'group_by' => function (ActorEntity $choice) {
                if ($choice->getFollowed()) {
                    return 'Suivi';
                }
                if ($choice->getActive()) {
                    return 'Actif';
                }

                return 'Inactif';
            },
            'choice_label' => 'displayName',
            'required' => false,
        ])
            ->add('unassigned', CheckboxType::class, [
            'required' => false,
        ])
            ->add('followed', CheckboxType::class, [
            'required' => false,
        ]);

        // Status filters
        $formBuilder->add('status', ChoiceType::class, [
            'required' => false,
            'choices' => ActivityEntity::VALID_STATUS,
            'choice_label' => function ($choice, $key, $value) {
                return StatusEnum::STATUS_NAME[$choice];
            },
        ])
            ->add('active', CheckboxType::class, [
            'required' => false,
        ])
            ->add('notClosed', CheckboxType::class, [
            'required' => false,
        ]);

        // Date filters
        $formBuilder->add('endAfter', DateType::class, [
            'required' => false,
            'widget' => 'single_text',
        ])->add('startBefore', DateType::class, [
            'required' => false,
            'widget' => 'single_text',
        ]);

        // Submit
        $formBuilder->add('submit', SubmitType::class)->add('reset', ResetType::class);
    }
}